/**************************************************************************
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  27.12.2017  mifi  First Version
**************************************************************************/
#if !defined(__FPGA_H__)
#define __FPGA_H__

/**************************************************************************
*  Includes
**************************************************************************/
#include <stdint.h>

/**************************************************************************
*  Global Definitions
**************************************************************************/

#define FPGA_FRQ_HZ     96000000

/*
 * FPGA component base address
 */
#define FPGA_TEST_BASE  0x000
#define FPGA_GPIO_BASE  0x100
#define FPGA_TIMER_BASE 0x200

/*
 * Test register
 */
#define TEST_REG_0      (FPGA_TEST_BASE + 0) 
#define TEST_REG_1      (FPGA_TEST_BASE + 1)
#define TEST_REG_2      (FPGA_TEST_BASE + 2)
#define TEST_REG_3      (FPGA_TEST_BASE + 3)
#define TEST_REG_4      (FPGA_TEST_BASE + 4)
#define TEST_REG_5      (FPGA_TEST_BASE + 5)
#define TEST_REG_6      (FPGA_TEST_BASE + 6)
#define TEST_REG_7      (FPGA_TEST_BASE + 7)

/*
 * GPIO register
 */
#define GPIO_OUT        (FPGA_GPIO_BASE + 0)
#define GPIO_IN         (FPGA_GPIO_BASE + 1)
#define GPIO_SET        (FPGA_GPIO_BASE + 2)
#define GPIO_CLR        (FPGA_GPIO_BASE + 3)

/*
 * Timer register
 */
#define TIMER_STATUS    (FPGA_TIMER_BASE + 0)
#define TIMER_CONTROL   (FPGA_TIMER_BASE + 1)
#define TIMER_PERIOD_0  (FPGA_TIMER_BASE + 2)
#define TIMER_PERIOD_1  (FPGA_TIMER_BASE + 3)
#define TIMER_PERIOD_2  (FPGA_TIMER_BASE + 4)
#define TIMER_PERIOD_3  (FPGA_TIMER_BASE + 5)

/**************************************************************************
*  Macro Definitions
**************************************************************************/

#define FPGARead2(_a,_b)  FPGAReadMulti(_a,_b,2)
#define FPGARead4(_a,_b)  FPGAReadMulti(_a,_b,4)


#define FPGAWrite2(_a,_b)  FPGAWriteMulti(_a,_b,2)
#define FPGAWrite4(_a,_b)  FPGAWriteMulti(_a,_b,4)

/**************************************************************************
*  Functions Definitions
**************************************************************************/

/*
 * Init
 */
void    FPGAInit (void);
void    FPGAResetSet (void);
void    FPGAResetClear (void);
int     FPGATest (void);


/*
 * Read/Write
 */
uint8_t FPGARead (uint16_t wAddress);
void    FPGAWrite (uint16_t wAddress, uint8_t bData);
void    FPGAReadMulti (uint16_t wAddress, void *pBuffer, uint8_t bCount);
void    FPGAWriteMulti (uint16_t wAddress, void *pBuffer, uint8_t bCount);

/*
 * Timer
 */
void    FPGATimerInit (uint16_t wMilliSec);
void    FPGATimerStart (void);
void    FPGATimerStop (void);

#endif /* !__FPGA_H__ */

/*** EOF ***/
